% fftserial interface v 2

%Sam Bobb, Daniel Cornew, Ryan Deeter
%ME 333 Lab 5: FFT of Analog Input, Winter 2010

%This code receives the single sided magnitude FFT calculated on the PIC
%and the raw samples through serial communication.  Matlab calculates its
%own FFT of the raw samples, then plots the input signal, PIC calculated
%FFT, and Matlab calculated FFT.




Fs = 1280;                     % Sampling frequency
T  = 1/Fs;                     % Sample time
L  = 256;                      % Length of signal
t  = (0:L-1)*T;                % Time vector
t2 = 0:255;                    % Time vector 1 to 1024


%for runtimes=1:100      %UNCOMMENT FOR CONTINUOUS FFT

columns = 9;             %how many columns of data

%check if the serial port already exists
%if it does, close it, it probably means something bad happened last run
if exist('COM','var')
    disp('closing old port instance');
    fclose(COM)
    delete(COM)
    clear COM
    disp('port closed');
end

% open the serial port
COM = serial('COM6');
set(COM,'BaudRate',19200);
fopen(COM)     

disp('port opened');

% send the command to print data
fprintf(COM,'p')


%there's usually some garbage at the beginning (like the echoed p 
%character command and line  breaks); this reads lines in until we find 
%the start line
text = fscanf(COM, '%s');
while isempty(strfind(text, 'START'))
    text = fscanf(COM, '%s');
end


% read number of rows
rowstext = fscanf(COM, '%s');
pat = 'ROWS=';
split = regexp(rowstext, pat, 'split');
rows = str2double(split(2));

DATA = zeros(rows,columns);

%generate the fscanf format argument
%repeate the formatchar character for the number of columns
formatchar = '%d';
serialpat = '';

%establishing fscanf format
for j = 1:columns
    serialpat = [serialpat ' ' formatchar];
end
    
%reads serial data and puts it into DATA
for j = 1:rows
    DATA(j, :) = fscanf(COM, serialpat, [columns 1]);
    %disp(COM.BytesAvailable)

end

%reads last line
last = fscanf(COM, '%s');

%verifies that last line read is the last line
if strcmp(last,'END')
    disp('Success')
else
    disp('Problem')
end

%disp(COM.BytesAvailable)       %for debugging

%plotting input signal
subplot(4, 1, 1)
plot(DATA(:,3))
title('Time Domain Signal')
xlabel('Time')
ylabel('Signal')

%calculating FFT in Matlab
NFFT = 2^nextpow2(L); % Next power of 2 from length of y
Y = fft(DATA(:,3),NFFT)/L;

%Y = DATA(:,1) + DATA(:,2)*1i;

f = Fs/2*linspace(0,1,NFFT/2+1);

% Plot Matlab's and PIC's single-sided amplitude spectrum.
subplot(4, 1, 2)
plot(f,abs(Y(1:NFFT/2+1)),DATA(:,5),DATA(:,6))
legend('MATLAB', 'PIC32')
title('Single Sided FFT Amplitude')
xlabel('Frequency (Hz)')
ylabel('|Y(f)|')


% Plot Phase
for j = 1:(NFFT/2+1)
    if abs(Y(j)) > abs(mean(Y(1:NFFT/2+1)))
        FFTPhase(j) = atan2(imag(Y(j)),real(Y(j)));
    else
        FFTPhase(j) = 0;
    end
end
subplot(4, 1, 3)
plot(1:(NFFT/2+1),FFTPhase,1:256,DATA(:,9))
legend('MATLAB','PIC32')
title('FFT Phase')
xlabel('Frequency')
ylabel('Phase Shift')


% Plot IFFTs
subplot(4, 1, 4)
plot(t2,fft(conj(Y)),t2,DATA(:,1))
legend('MATLAB','PIC32')
title('Inverse FFT')
xlabel('Time')
ylabel('Signal')

%pause(2)        %UNCOMMENT FOR CONTINOUS FFT

%end             %UNCOMMENT FOR CONTINUOUS FFT

%closes serial port
fclose(COM)
delete(COM)
clear COM
disp('port closed');
